/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2008 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Germain Haugou
 */

package org.objectweb.fractal.cecilia.adl.dynamic;

import java.util.Map;

import org.objectweb.fractal.adl.Factory;

public class Launcher extends org.objectweb.fractal.cecilia.adl.Launcher {

  /**
   * The name of the default ADL of the compiler used when compiling a standard
   * component.
   */
  public static final String CECILIADYN_BASIC_COMPILER_ADL = "org.objectweb.fractal.cecilia.adl.CeciliaRemoteFactory";

  /**
   * The name of the ADL of the compiler used when compiling a dynamically
   * loadable component.
   */
  public static final String CECILIADYN_DYNAMIC_COMPILER_ADL = "org.objectweb.fractal.cecilia.adl.CeciliaExportBindDynamicFactory";

  protected CmdFlag dynamicOpt;

  public Launcher(Map<String, String> options)
      throws InvalidCommandLineException, CompilerInstantiationException {
    super(options);
  }

  public Launcher(String outDir, String sources)
      throws InvalidCommandLineException, CompilerInstantiationException {
    super(outDir, sources);
  }

  public Launcher(String... args) throws Exception {
    super(args);
  }

  protected void addOptions() {
    dynamicOpt = new CmdFlag("d", "dynamic",
        "Compile a dynamically loadable component");

    options.addOptions(targetDescOpt, compilerCmdOpt,
        cFlagsOpt, includePathOpt, linkerCmdOpt, ldFlagsOpt, ldPathOpt,
        archiverCmdOpt, linkerScriptOpt, executableNameCmdOpt,
        concurrentJobCmdOpt, printStackTraceOpt, dynamicOpt);
  }

  protected void init(final String... args) throws InvalidCommandLineException,
      CompilerInstantiationException {
    super.init(args);
    
    String cFlags = "-DTYPEDINTERFACE=1";
    if (compilerContext.get(C_FLAGS) != null)
      cFlags += " " + compilerContext.get(C_FLAGS);
    compilerContext.put(C_FLAGS, cFlags);
  }

  protected Factory createCompiler(final Map<Object, Object> compilerContext,
      final CommandLine cmdLine) throws CompilerInstantiationException {
    final String compilerAdlDefinition;
    if (dynamicOpt.isPresent(cmdLine))
      compilerAdlDefinition = CECILIADYN_DYNAMIC_COMPILER_ADL;
    else
      compilerAdlDefinition = CECILIADYN_BASIC_COMPILER_ADL;

    return createCompiler(compilerAdlDefinition, compilerContext);
  }

  public static void main(final String... args) {
    try {
      new Launcher(args);
    } catch (final Exception e) {
      // never append
      e.printStackTrace();
    }
  }

}
